@@ -99,7 +99,7 @@ glue_times_redirect2_helper<true>::apply(Mat<typename T1::elem_type>& out, const
     arma_debug_assert_mul_size(A, B, "matrix multiplication");
     
     #if defined(ARMA_OPTIMISE_SOLVE_SYMPD)
-      const bool status = (strip_inv<T1>::do_inv_sympd) ? auxlib::solve_sympd_fast(out, A, B, true) : auxlib::solve_square_fast(out, A, B);
+      const bool status = (strip_inv<T1>::do_inv_sympd) ? auxlib::solve_sympd_fast(out, A, B) : auxlib::solve_square_fast(out, A, B);
     #else
       const bool status = auxlib::solve_square_fast(out, A, B);
     #endif
@@ -115,7 +115,7 @@ glue_times_redirect2_helper<true>::apply(Mat<typename T1::elem_type>& out, const
   
   #if defined(ARMA_OPTIMISE_SOLVE_SYMPD)
     {
-    if( (strip_inv<T2>::do_inv_sympd) && (is_cx<eT>::no) )
+    if(strip_inv<T2>::do_inv_sympd)
       {
       // replace A*inv_sympd(B) with trans( solve(trans(B),trans(A)) )
       // transpose of B is avoided as B is explicitly marked as symmetric
@@ -132,7 +132,7 @@ glue_times_redirect2_helper<true>::apply(Mat<typename T1::elem_type>& out, const
       
       arma_debug_assert_mul_size(At.n_cols, At.n_rows, B.n_rows, B.n_cols, "matrix multiplication");
       
-      const bool status = auxlib::solve_sympd_fast(out, B, At, true);
+      const bool status = auxlib::solve_sympd_fast(out, B, At);
       
       if(status == false)
         {
@@ -256,7 +256,7 @@ glue_times_redirect3_helper<true>::apply(Mat<typename T1::elem_type>& out, const
     arma_debug_assert_mul_size(A, BC, "matrix multiplication");
     
     #if defined(ARMA_OPTIMISE_SOLVE_SYMPD)
-      const bool status = (strip_inv<T1>::do_inv_sympd) ? auxlib::solve_sympd_fast(out, A, BC, true) : auxlib::solve_square_fast(out, A, BC);
+      const bool status = (strip_inv<T1>::do_inv_sympd) ? auxlib::solve_sympd_fast(out, A, BC) : auxlib::solve_square_fast(out, A, BC);
     #else
       const bool status = auxlib::solve_square_fast(out, A, BC);
     #endif
@@ -291,7 +291,7 @@ glue_times_redirect3_helper<true>::apply(Mat<typename T1::elem_type>& out, const
     Mat<eT> solve_result;
     
     #if defined(ARMA_OPTIMISE_SOLVE_SYMPD)
-      const bool status = (strip_inv<T2>::do_inv_sympd) ? auxlib::solve_sympd_fast(solve_result, B, C, true) : auxlib::solve_square_fast(solve_result, B, C);
+      const bool status = (strip_inv<T2>::do_inv_sympd) ? auxlib::solve_sympd_fast(solve_result, B, C) : auxlib::solve_square_fast(solve_result, B, C);
     #else
       const bool status = auxlib::solve_square_fast(solve_result, B, C);
     #endif
