@@ -28,24 +28,20 @@ spglue_times::apply(SpMat<typename T1::elem_type>& out, const SpGlue<T1,T2,spglu
   
   typedef typename T1::elem_type eT;
   
-  // unconditionally unwrapping, as the column iterator in SpSubview is slow
+  const unwrap_spmat<T1> UA(X.A);
+  const unwrap_spmat<T2> UB(X.B);
   
-  const unwrap_spmat<T1> tmp1(X.A);
-  const unwrap_spmat<T2> tmp2(X.B);
-  
-  const SpProxy<typename unwrap_spmat<T1>::stored_type> pa(tmp1.M);
-  const SpProxy<typename unwrap_spmat<T2>::stored_type> pb(tmp2.M);
-  
-  const bool is_alias = pa.is_alias(out) || pb.is_alias(out);
+  const bool is_alias = (UA.is_alias(out) || UB.is_alias(out));
   
   if(is_alias == false)
     {
-    spglue_times::apply_noalias(out, pa, pb);
+    spglue_times::apply_noalias(out, UA.M, UB.M);
     }
   else
     {
     SpMat<eT> tmp;
-    spglue_times::apply_noalias(tmp, pa, pb);
+    
+    spglue_times::apply_noalias(tmp, UA.M, UB.M);
     
     out.steal_mem(tmp);
     }
@@ -53,18 +49,50 @@ spglue_times::apply(SpMat<typename T1::elem_type>& out, const SpGlue<T1,T2,spglu
 
 
 
-template<typename eT, typename T1, typename T2>
-arma_hot
+template<typename T1, typename T2>
 inline
 void
-spglue_times::apply_noalias(SpMat<eT>& c, const SpProxy<T1>& pa, const SpProxy<T2>& pb)
+spglue_times::apply(SpMat<typename T1::elem_type>& out, const SpGlue<SpOp<T1,spop_scalar_times>,T2,spglue_times>& X)
   {
   arma_extra_debug_sigprint();
   
-  const uword x_n_rows = pa.get_n_rows();
-  const uword x_n_cols = pa.get_n_cols();
-  const uword y_n_rows = pb.get_n_rows();
-  const uword y_n_cols = pb.get_n_cols();
+  typedef typename T1::elem_type eT;
+  
+  const unwrap_spmat<T1> UA(X.A.m);
+  const unwrap_spmat<T2> UB(X.B);
+  
+  const bool is_alias = (UA.is_alias(out) || UB.is_alias(out));
+  
+  if(is_alias == false)
+    {
+    spglue_times::apply_noalias(out, UA.M, UB.M);
+    }
+  else
+    {
+    SpMat<eT> tmp;
+    
+    spglue_times::apply_noalias(tmp, UA.M, UB.M);
+    
+    out.steal_mem(tmp);
+    }
+  
+  out *= X.A.aux;
+  }
+
+
+
+template<typename eT>
+arma_hot
+inline
+void
+spglue_times::apply_noalias(SpMat<eT>& c, const SpMat<eT>& x, const SpMat<eT>& y)
+  {
+  arma_extra_debug_sigprint();
+  
+  const uword x_n_rows = x.n_rows;
+  const uword x_n_cols = x.n_cols;
+  const uword y_n_rows = y.n_rows;
+  const uword y_n_cols = y.n_cols;
 
   arma_debug_assert_mul_size(x_n_rows, x_n_cols, y_n_rows, y_n_cols, "matrix multiplication");
 
@@ -80,8 +108,8 @@ spglue_times::apply_noalias(SpMat<eT>& c, const SpProxy<T1>& pa, const SpProxy<T
   //SpMat<typename T1::elem_type> c(x_n_rows, y_n_cols); // Initializes col_ptrs to 0.
   c.zeros(x_n_rows, y_n_cols);
   
-  //if( (pa.get_n_elem() == 0) || (pb.get_n_elem() == 0) )
-  if( (pa.get_n_nonzero() == 0) || (pb.get_n_nonzero() == 0) )
+  //if( (x.n_elem == 0) || (y.n_elem == 0) )
+  if( (x.n_nonzero == 0) || (y.n_nonzero == 0) )
     {
     return;
     }
@@ -90,8 +118,8 @@ spglue_times::apply_noalias(SpMat<eT>& c, const SpProxy<T1>& pa, const SpProxy<T
   podarray<uword> index(x_n_rows);
   index.fill(x_n_rows); // Fill with invalid links.
   
-  typename SpProxy<T2>::const_iterator_type y_it  = pb.begin();
-  typename SpProxy<T2>::const_iterator_type y_end = pb.end();
+  typename SpMat<eT>::const_iterator y_it  = y.begin();
+  typename SpMat<eT>::const_iterator y_end = y.end();
 
   // SYMBMM: calculate column pointers for resultant matrix to obtain a good
   // upper bound on the number of nonzero elements.
@@ -102,7 +130,7 @@ spglue_times::apply_noalias(SpMat<eT>& c, const SpProxy<T1>& pa, const SpProxy<T
     const uword y_it_row = y_it.row();
     
     // Look through the column that this point (*y_it) could affect.
-    typename SpProxy<T1>::const_iterator_type x_it = pa.begin_col(y_it_row);
+    typename SpMat<eT>::const_iterator x_it = x.begin_col_no_sync(y_it_row);
     
     while(x_it.col() == y_it_row)
       {
@@ -155,9 +183,7 @@ spglue_times::apply_noalias(SpMat<eT>& c, const SpProxy<T1>& pa, const SpProxy<T
   podarray<eT> sums(x_n_rows); // Partial sums.
   sums.zeros();
   
-  // setting the size of 'sorted_indices' to x_n_rows is a better-than-nothing guess;
-  // the correct minimum size is determined later
-  podarray<uword> sorted_indices(x_n_rows);
+  podarray<uword> sorted_indices(x_n_rows);  // upper bound
   
   // last_ind is already set to x_n_rows, and cur_col_length is already set to 0.
   // We will loop through all columns as necessary.
@@ -182,7 +208,7 @@ spglue_times::apply_noalias(SpMat<eT>& c, const SpProxy<T1>& pa, const SpProxy<T
     access::rw(c.col_ptrs[cur_col]) = cur_pos;
 
     // Check all elements in this column.
-    typename SpProxy<T2>::const_iterator_type y_col_it = pb.begin_col(cur_col);
+    typename SpMat<eT>::const_iterator y_col_it = y.begin_col_no_sync(cur_col);
     
     while(y_col_it.col() == cur_col)
       {
@@ -190,7 +216,7 @@ spglue_times::apply_noalias(SpMat<eT>& c, const SpProxy<T1>& pa, const SpProxy<T
       
       // Check all elements in the column of the other matrix corresponding to
       // the row of this column.
-      typename SpProxy<T1>::const_iterator_type x_col_it = pa.begin_col(y_col_it_row);
+      typename SpMat<eT>::const_iterator x_col_it = x.begin_col_no_sync(y_col_it_row);
 
       const eT y_value = (*y_col_it);
 
@@ -217,14 +243,6 @@ spglue_times::apply_noalias(SpMat<eT>& c, const SpProxy<T1>& pa, const SpProxy<T
       }
 
     // Now sort the indices that were used in this column.
-    //podarray<uword> sorted_indices(c.col_ptrs[cur_col + 1] - c.col_ptrs[cur_col]);
-    sorted_indices.set_min_size(c.col_ptrs[cur_col + 1] - c.col_ptrs[cur_col]);
-    
-    // .set_min_size() can only enlarge the array to the specified size,
-    // hence if we request a smaller size than already allocated,
-    // no new memory allocation is done
-    
-    
     uword cur_index = 0;
     while(last_ind != x_n_rows + 1)
       {
@@ -268,43 +286,6 @@ spglue_times::apply_noalias(SpMat<eT>& c, const SpProxy<T1>& pa, const SpProxy<T
 
 
 
-//
-//
-// spglue_times2: scalar*(A * B)
-
-
-
-template<typename T1, typename T2>
-inline
-void
-spglue_times2::apply(SpMat<typename T1::elem_type>& out, const SpGlue<T1,T2,spglue_times2>& X)
-  {
-  arma_extra_debug_sigprint();
-  
-  typedef typename T1::elem_type eT;
-  
-  const SpProxy<T1> pa(X.A);
-  const SpProxy<T2> pb(X.B);
-  
-  const bool is_alias = pa.is_alias(out) || pb.is_alias(out);
-  
-  if(is_alias == false)
-    {
-    spglue_times::apply_noalias(out, pa, pb);
-    }
-  else
-    {
-    SpMat<eT> tmp;
-    spglue_times::apply_noalias(tmp, pa, pb);
-    
-    out.steal_mem(tmp);
-    }
-  
-  out *= X.aux;
-  }
-
-
-
 //
 //
 //
@@ -351,12 +332,17 @@ spglue_times_misc::sparse_times_dense(Mat<typename T1::elem_type>& out, const T1
       
       if(A_n_rows == B_n_cols)
         {
-        out = Bt * At;
-        out = strans(out);  // since 'out' is square-sized, this will do an inplace transpose
+        spglue_times_misc::dense_times_sparse(out, Bt, At);
+        
+        op_strans::apply_mat(out, out);  // since 'out' is square-sized, this will do an inplace transpose
         }
       else
         {
-        out = strans(Bt * At);
+        Mat<eT> tmp;
+        
+        spglue_times_misc::dense_times_sparse(tmp, Bt, At);
+        
+        op_strans::apply_mat(out, tmp);
         }
       }
     else
@@ -437,7 +423,7 @@ spglue_times_misc::dense_times_sparse(Mat<typename T1::elem_type>& out, const T1
             const uword col_offset_delta = col_offset_2 - col_offset_1;
             
             const uvec    indices(const_cast<uword*>(&(Y.row_indices[col_offset_1])), col_offset_delta, false, false);
-            const Col<eT>   Y_col(const_cast<   eT*>(&(Y.values[col_offset_1])     ), col_offset_delta, false, false);
+            const Col<eT>   Y_col(const_cast<   eT*>(&(     Y.values[col_offset_1])), col_offset_delta, false, false);
             
             out.col(i) = X.cols(indices) * Y_col;
             }
@@ -475,4 +461,223 @@ spglue_times_misc::dense_times_sparse(Mat<typename T1::elem_type>& out, const T1
 
 
 
+//
+
+
+
+template<typename T1, typename T2>
+inline
+void
+spglue_times_mixed::sparse_times_sparse(SpMat< typename promote_type<typename T1::elem_type, typename T2::elem_type>::result >& out, const T1& X, const T2& Y)
+  {
+  arma_extra_debug_sigprint();
+  
+  typedef typename T1::elem_type eT1;
+  typedef typename T2::elem_type eT2;
+  
+  typedef typename promote_type<eT1,eT2>::result out_eT;
+  
+  promote_type<eT1,eT2>::check();
+  
+  if( (is_same_type<eT1,out_eT>::no) && (is_same_type<eT2,out_eT>::yes) )
+    {
+    // upgrade T1
+    
+    const unwrap_spmat<T1> UA(X);
+    const unwrap_spmat<T2> UB(Y);
+    
+    const SpMat<eT1>& A = UA.M;
+    const SpMat<eT2>& B = UB.M;
+    
+    SpMat<out_eT> AA(arma_layout_indicator(), A);
+    
+    for(uword i=0; i < A.n_nonzero; ++i)  { access::rw(AA.values[i]) = out_eT(A.values[i]); }
+    
+    const SpMat<out_eT>& BB = reinterpret_cast< const SpMat<out_eT>& >(B);
+    
+    spglue_times::apply_noalias(out, AA, BB);
+    }
+  else
+  if( (is_same_type<eT1,out_eT>::yes) && (is_same_type<eT2,out_eT>::no) )
+    {
+    // upgrade T2 
+    
+    const unwrap_spmat<T1> UA(X);
+    const unwrap_spmat<T2> UB(Y);
+    
+    const SpMat<eT1>& A = UA.M;
+    const SpMat<eT2>& B = UB.M;
+    
+    const SpMat<out_eT>& AA = reinterpret_cast< const SpMat<out_eT>& >(A);
+    
+    SpMat<out_eT> BB(arma_layout_indicator(), B);
+    
+    for(uword i=0; i < B.n_nonzero; ++i)  { access::rw(BB.values[i]) = out_eT(B.values[i]); }
+    
+    spglue_times::apply_noalias(out, AA, BB);
+    }
+  else
+    {
+    // upgrade T1 and T2
+    
+    const unwrap_spmat<T1> UA(X);
+    const unwrap_spmat<T2> UB(Y);
+    
+    const SpMat<eT1>& A = UA.M;
+    const SpMat<eT2>& B = UB.M;
+    
+    SpMat<out_eT> AA(arma_layout_indicator(), A);
+    SpMat<out_eT> BB(arma_layout_indicator(), B);
+    
+    for(uword i=0; i < A.n_nonzero; ++i)  { access::rw(AA.values[i]) = out_eT(A.values[i]); }
+    for(uword i=0; i < B.n_nonzero; ++i)  { access::rw(BB.values[i]) = out_eT(B.values[i]); }
+    
+    spglue_times::apply_noalias(out, AA, BB);
+    }
+  }
+
+
+
+template<typename T1, typename T2>
+inline
+void
+spglue_times_mixed::sparse_times_dense(Mat< typename promote_type<typename T1::elem_type, typename T2::elem_type>::result >& out, const T1& X, const T2& Y)
+  {
+  arma_extra_debug_sigprint();
+  
+  typedef typename T1::elem_type eT1;
+  typedef typename T2::elem_type eT2;
+  
+  typedef typename promote_type<eT1,eT2>::result out_eT;
+  
+  promote_type<eT1,eT2>::check();
+  
+  if( (is_same_type<eT1,out_eT>::no) && (is_same_type<eT2,out_eT>::yes) )
+    {
+    // upgrade T1
+    
+    const unwrap_spmat<T1> UA(X);
+    const quasi_unwrap<T2> UB(Y);
+    
+    const SpMat<eT1>& A = UA.M;
+    const   Mat<eT2>& B = UB.M;
+    
+    SpMat<out_eT> AA(arma_layout_indicator(), A);
+    
+    for(uword i=0; i < A.n_nonzero; ++i)  { access::rw(AA.values[i]) = out_eT(A.values[i]); }
+    
+    const Mat<out_eT>& BB = reinterpret_cast< const Mat<out_eT>& >(B);
+    
+    spglue_times_misc::sparse_times_dense(out, AA, BB);
+    }
+  else
+  if( (is_same_type<eT1,out_eT>::yes) && (is_same_type<eT2,out_eT>::no) )
+    {
+    // upgrade T2 
+    
+    const unwrap_spmat<T1> UA(X);
+    const quasi_unwrap<T2> UB(Y);
+    
+    const SpMat<eT1>& A = UA.M;
+    const   Mat<eT2>& B = UB.M;
+    
+    const SpMat<out_eT>& AA = reinterpret_cast< const SpMat<out_eT>& >(A);
+    
+    const Mat<out_eT> BB = conv_to< Mat<out_eT> >::from(B);
+    
+    spglue_times_misc::sparse_times_dense(out, AA, BB);
+    }
+  else
+    {
+    // upgrade T1 and T2
+    
+    const unwrap_spmat<T1> UA(X);
+    const quasi_unwrap<T2> UB(Y);
+    
+    const SpMat<eT1>& A = UA.M;
+    const   Mat<eT2>& B = UB.M;
+    
+    SpMat<out_eT> AA(arma_layout_indicator(), A);
+    
+    for(uword i=0; i < A.n_nonzero; ++i)  { access::rw(AA.values[i]) = out_eT(A.values[i]); }
+    
+    const Mat<out_eT> BB = conv_to< Mat<out_eT> >::from(B);
+    
+    spglue_times_misc::sparse_times_dense(out, AA, BB);
+    }
+  }
+
+
+
+template<typename T1, typename T2>
+inline
+void
+spglue_times_mixed::dense_times_sparse(Mat< typename promote_type<typename T1::elem_type, typename T2::elem_type>::result >& out, const T1& X, const T2& Y)
+  {
+  arma_extra_debug_sigprint();
+  
+  typedef typename T1::elem_type eT1;
+  typedef typename T2::elem_type eT2;
+  
+  typedef typename promote_type<eT1,eT2>::result out_eT;
+  
+  promote_type<eT1,eT2>::check();
+  
+  if( (is_same_type<eT1,out_eT>::no) && (is_same_type<eT2,out_eT>::yes) )
+    {
+    // upgrade T1
+    
+    const quasi_unwrap<T1> UA(X);
+    const unwrap_spmat<T2> UB(Y);
+    
+    const   Mat<eT1>& A = UA.M;
+    const SpMat<eT2>& B = UB.M;
+    
+    const Mat<out_eT> AA = conv_to< Mat<out_eT> >::from(A);
+    
+    const SpMat<out_eT>& BB = reinterpret_cast< const SpMat<out_eT>& >(B);
+    
+    spglue_times_misc::dense_times_sparse(out, AA, BB);
+    }
+  else
+  if( (is_same_type<eT1,out_eT>::yes) && (is_same_type<eT2,out_eT>::no) )
+    {
+    // upgrade T2 
+    
+    const quasi_unwrap<T1> UA(X);
+    const unwrap_spmat<T2> UB(Y);
+    
+    const   Mat<eT1>& A = UA.M;
+    const SpMat<eT2>& B = UB.M;
+    
+    const Mat<out_eT>& AA = reinterpret_cast< const Mat<out_eT>& >(A);
+    
+    SpMat<out_eT> BB(arma_layout_indicator(), B);
+    
+    for(uword i=0; i < B.n_nonzero; ++i)  { access::rw(BB.values[i]) = out_eT(B.values[i]); }
+    
+    spglue_times_misc::dense_times_sparse(out, AA, BB);
+    }
+  else
+    {
+    // upgrade T1 and T2
+    
+    const quasi_unwrap<T1> UA(X);
+    const unwrap_spmat<T2> UB(Y);
+    
+    const   Mat<eT1>& A = UA.M;
+    const SpMat<eT2>& B = UB.M;
+    
+    const Mat<out_eT> AA = conv_to< Mat<out_eT> >::from(A);
+    
+    SpMat<out_eT> BB(arma_layout_indicator(), B);
+    
+    for(uword i=0; i < B.n_nonzero; ++i)  { access::rw(BB.values[i]) = out_eT(B.values[i]); }
+    
+    spglue_times_misc::dense_times_sparse(out, AA, BB);
+    }
+  }
+
+
+
 //! @}
